package sacooliveros.whatsappweb.api;

import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeoutException;
import java.util.logging.Logger;
import org.apache.catalina.connector.Response;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import it.auties.whatsapp.api.Whatsapp;
import sacooliveros.whatsappweb.bean.WhatsappBean;
import sacooliveros.whatsappweb.service.WhatsappService;
import sacooliveros.whatsappweb.utils.WhatsappUtils;

@RestController
@CrossOrigin("*")
@RequestMapping("api/v1")
public class WhatsappRest {

    Logger logger = Logger.getLogger(WhatsappRest.class.getName());
    WhatsappBean whatsappBean;
    WhatsappService whatsappService;

    public WhatsappRest(WhatsappBean whatsappBean, WhatsappService whatsappService) {
        this.whatsappBean = whatsappBean;
        this.whatsappService = whatsappService;
    }

    @GetMapping("qr")
    public ResponseEntity<?>  getQR() throws ExecutionException, InterruptedException, TimeoutException {
        Whatsapp w = whatsappBean.getWhatsapp();
        WhatsappUtils utils = new WhatsappUtils();

        if (w == null) {
            return ResponseEntity.status(Response.SC_OK).body(utils.generateQR());
        }

        w.connect().get(5, TimeUnit.SECONDS);
        if (w.isConnected()) {
            return ResponseEntity.status(Response.SC_OK).body("Already connected");
        }

        try {
            w.connect().get(5, TimeUnit.SECONDS);
        } catch (InterruptedException | ExecutionException | TimeoutException e) {
            logger.severe(e.getMessage());
            return ResponseEntity.status(Response.SC_INTERNAL_SERVER_ERROR).body("Connection failed");
        }

        if (!w.isConnected()) {
            return ResponseEntity.status(Response.SC_INTERNAL_SERVER_ERROR).body("Connection failed");
        }else{
            return ResponseEntity.status(Response.SC_INTERNAL_SERVER_ERROR).body("Try logging out first");
        }
    }

    @GetMapping("logout")
    public ResponseEntity<?>  logout() {
        Whatsapp w = whatsappBean.getWhatsapp();
        if (w == null) {
            return ResponseEntity.status(Response.SC_OK).body("Not connected");
        }

        try {
            w.disconnect().get(5, TimeUnit.SECONDS);
            w.logout().get(5, TimeUnit.SECONDS);
            return ResponseEntity.status(Response.SC_OK).body("Logged out");
        } catch (InterruptedException | ExecutionException | TimeoutException e) {
            logger.severe(e.getMessage());
            return ResponseEntity.status(Response.SC_INTERNAL_SERVER_ERROR).body("Logout failed");
        }
    }

    @PostMapping(value = "send", consumes = MediaType.MULTIPART_FORM_DATA_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<?> send(@RequestParam Map<String, String> payload, @RequestParam Map<String, MultipartFile> files) {
        Whatsapp w = whatsappBean.getWhatsapp();
        if (w == null) {
            return ResponseEntity.status(Response.SC_OK).body("Not connected");
        }
        try{
            w.reconnect().get(5, TimeUnit.SECONDS);
            System.out.println(w.store().about());
            if (w.isConnected()) {
                whatsappService.send(payload, files, w);
                return ResponseEntity.status(Response.SC_OK).body(payload);
            }else{
                return ResponseEntity.status(Response.SC_OK).body("Not connected");
            }
        }catch (Exception e){
            logger.severe(e.getMessage());
            return ResponseEntity.status(Response.SC_INTERNAL_SERVER_ERROR).body("Send failed");
        }
    }

}
